#!/bin/bash

function init_vars() {
 VERSIONFILE=WCS.version
 FLASHPHONER_URL="my.flashphoner.com"
 FLASHPHONER_SILENT=$1
 INSTALLER_DIR=`pwd`
 PRODUCT="FlashphonerWebCallServer"
 VERSION=`cat ./$VERSIONFILE`
 SUPPORT_URL="http://flashphoner.com"
 FLASHPHONER_WCS_HOME_FULL="/usr/local/$PRODUCT-$VERSION"
 FLASHPHONER_WCS_HOME="/usr/local/$PRODUCT"
 NATIVE_LIB_DIR=$INSTALLER_DIR/server/lib/so
 LOCAL_LIB_DIR=/usr/local/lib
 PRODUCT_CONFIG=$INSTALLER_DIR/server/conf/flashphoner.properties
 IP_LOCAL=$2
 IP_EXTERNAL=$3
 PRODUCT_MAJOR_VERSION=`echo $VERSION | cut -d. -f1`
 PRODUCT_MINOR_VERSION=`echo $VERSION | cut -d. -f2`

 MAIN_PID='FlashphonerMainWebCallServer.pid'
 PID_PATH='/var/run'

 ALREADY_INSTALLED=false
 PRODUCT_VERSION_FILE=$FLASHPHONER_WCS_HOME/conf/$VERSIONFILE
 PREVIOUS_MAJOR_VERSION=0
 PREVIOUS_MINOR_VERSION=0
 UPDATE_TO_NON_ROOT=false
 WCS_FD_LIMIT=""

 FLASHPHONER_USER='flashphoner'
 USER_EXISTS=true

 DAEMON_TYPE=`ps --no-headers -o comm 1`

 JAVA_VERSION=0
 JDK_VERSION=""
 JVM_ARCH_DIGIT=""
 
 HUGEPAGES_MOUNT_POINT=""

 SYSTEMD_PATH=/etc/systemd/system
 INITD_PATH=/etc/init.d
 SYSTEMD_FILE=$SYSTEMD_PATH/webcallserver.service
 INITD_FILE=$INITD_PATH/webcallserver
}

function inform_user {
 if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
  echo "$1"
 fi
}

function check_user {
 if [[ $(getent passwd $FLASHPHONER_USER) == "" ]]; then
  USER_EXISTS=false
 else
  USER_EXISTS=true
 fi
}

function check_server_state {
 if [[ -f $FLASHPHONER_WCS_HOME/bin/setenv.sh && $(grep "WCS_NON_ROOT=true" $FLASHPHONER_WCS_HOME/bin/setenv.sh) ]]; then
  PID_PATH="$FLASHPHONER_WCS_HOME/bin"
 fi
 if [[ -f $PID_PATH/$MAIN_PID && $(pgrep -F $PID_PATH/$MAIN_PID) ]]; then
  echo -e "\nYou have to stop $PRODUCT before update\n"
  read -p "Stop now [yes/no] ? " confirm
  if [[ $confirm = y* ]]; then
   if [[ $DAEMON_TYPE == 'systemd' ]]; then
    systemctl stop webcallserver
   else 
    service webcallserver stop
   fi  
   if [[ -f $PID_PATH/$MAIN_PID && $(pgrep -F $PID_PATH/$MAIN_PID) ]]; then
    echo -e "Service did not stop, stopping application\n"
    $FLASHPHONER_WCS_HOME/bin/webcallserver stop
   fi
   echo -e "\n"
  else
   echo -e "\nAbort updating\n "
   exit 1
  fi
 fi
}

function detect_ip {
 extIP1=`/usr/bin/curl -s https://api.ipify.org ; echo`
 extIP2=`/usr/bin/curl -s ipinfo.io/ip`
 localIP=""
 if [[ $extIP1 == '' && $extIP2 == '' ]]; then
  extIP="undefined"	
 elif [[ $extIP1 != '' ]]; then
  extIP=$extIP1
 else
  extIP=$extIP2
 fi
 localIPcount=0
 for i in $(hostname -I) ; do
  ((localIPcount++))
  if [[ "$extIP" == "$i" ]]; then		
   localIP=$extIP
  fi
 done
 if [ $localIPcount = 0 ]; then
  localIP="undefined"
 fi	

 if [[ "$extIP" == "$localIP" ]]; then
  IP_EXTERNAL=$extIP
  IP_LOCAL=$extIP
 else
  echo -e "We have found these IP on your server\n"
  echo -e "External IP\t\tLocal IPs" 
  echo -e "$extIP\t\t$(hostname -I)\n"
  read -p "Is your server located in your LAN [yes/no] ? " confirm
  if [[ $confirm = y* ]]; then
   set_local_ip
  else 
   if [[ $extIP != "undefined" ]]; then
    IP_EXTERNAL=$extIP
   fi	
   set_local_ip
  fi
  echo -e "\n"
  read -p "Press any key to continue..."	
 fi	
}	

function set_local_ip {
 if [ $localIPcount = 1 ]; then
  if [[ "$localIP" == "" ]]; then
   localIP=$(hostname -I | tr -d '[:space:]')
  fi	
  if [[ $IP_EXTERNAL == "" ]]; then
   IP_EXTERNAL=$localIP
  fi    
  IP_LOCAL=$localIP
 else
  if [[ "$localIP" == "undefined" ]];then
   read -p "What is your server LAN IP? " addr
  else 	
   read -p "We have found $localIPcount IPs, what should be used for WCS: $(hostname -I)? " addr
  fi  
  if [[ $IP_EXTERNAL == "" ]]; then
   IP_EXTERNAL=$addr
  fi    
  IP_LOCAL=$addr
 fi
 echo -e "Thanks. We have configured:\n"
 echo -e "ip = $IP_EXTERNAL"
 echo -e "ip_local = $IP_LOCAL"  	
}

function check_write_permissions() {
 if [ ! -w "/usr/local" ]; then	
  echo ""
  echo "ERROR: Write to /usr/local restricted for user running install.sh"
  echo ""
  exit 1;
 fi
}


function is_server_installed() {
 if [ -f $PRODUCT_VERSION_FILE ]; then
  PREVIOUS_VERSION=`cat $PRODUCT_VERSION_FILE`
  ALREADY_INSTALLED=true
  PREVIOUS_MAJOR_VERSION=`echo $PREVIOUS_VERSION | cut -d. -f1`
  PREVIOUS_MINOR_VERSION=`echo $PREVIOUS_VERSION | cut -d. -f2`
 fi

 if [ "$PREVIOUS_VERSION" = $VERSION ]; then
  echo "******************************************************"
  echo "*                                                    *"
  echo "*    Flashphoner v.$VERSION already installed        *"
  echo "*                                                    *"
  echo "******************************************************"
  read cont < /dev/tty
  exit 1;
 fi

 inform_user "******************************************************
*                                                    *"

 if [ $ALREADY_INSTALLED != true ]; then
  inform_user "*       Installing Flashphoner v.$VERSION"
 else
  inform_user "*  Update Flashphoner from $PREVIOUS_VERSION to $VERSION"  
 fi
 inform_user "*                                                    *
*  (C) Flashphoner.com 2010. All rights reserved.    *"
 if [ $ALREADY_INSTALLED != true ]; then
  inform_user "*  To install press ENTER, to abort press CTRL+C.    *"
 else
  check_server_state 
 fi
  inform_user "*                                                    *
*                                        Press ENTER *
******************************************************"

 if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
  read cont < /dev/tty
 fi
}

function get_fd_limit() {
 # Extract open files limit from previous settings to add to service #WCS-2913
 STEPS=("setenv.sh_env" "webcallserver_env" "webcallserver.service_param" "webcallserver_ulimit" "explicit")
 WCS_FD_LIMIT=""
 for step in ${STEPS[@]}; do
  case "$step" in
   setenv.sh_env)
    WCS_FD_LIMIT=$(grep -E "^WCS_FD_LIMIT" $FLASHPHONER_WCS_HOME/bin/setenv.sh | cut -d"=" -f 2)
    ;;
   webcallserver_env)
    WCS_FD_LIMIT=$(grep -E "[[:space:]]WCS_FD_LIMIT" $FLASHPHONER_WCS_HOME/bin/webcallserver | sed -e 's/^[[:space:]]*//' | cut -d"=" -f 2)
    ;;
   webcallserver.service_param)
    WCS_FD_LIMIT=$(grep -E "^LimitNOFILE" /etc/systemd/system/webcallserver.service | cut -d"=" -f 2)
    ;;
   webcallserver_ulimit)
    WCS_FD_LIMIT=$(grep "ulimit -n" $FLASHPHONER_WCS_HOME/bin/webcallserver | sed -e 's/^[[:space:]]*//' | cut -d" " -f 3)
    ;;
   explicit)
    WCS_FD_LIMIT=20000
    ;;
  esac
  if [[ $WCS_FD_LIMIT != "" && $WCS_FD_LIMIT =~ ^[[:digit:]] ]]; then
   break
  fi
 done
}


function get_hugepages_config() {
 WCS_CORE_CONF="$FLASHPHONER_WCS_HOME_FULL/conf/wcs-core.properties"
 KEYWORD="ZPath"
 # Since JDK 15 ZPath renamed to AllocateHeapAt #WCS-3094
 if [[ $JAVA_VERSION -ge 15 ]]; then
   KEYWORD="AllocateHeapAt"
 fi
 ZPATH_CONFIG=`grep -Eo "$KEYWORD=.*" $WCS_CORE_CONF`
 if [[ $ZPATH_CONFIG =~ "$KEYWORD=" ]]; then
  HUGEPAGES_MOUNT_POINT=`echo $ZPATH_CONFIG | cut -d" " -f 1 | cut -d"=" -f 2`
 fi
}

function check_java() {
 # Checking java
 inform_user "DETECTING java command..."

 JAVA_SYMLINK_BIN=`which java 2>/dev/null`
 if ! test -f "$JAVA_SYMLINK_BIN" ; then
  inform_user "
  ERROR: The Java command (java) could not be found.
  Search path: $PATH
  In most cases this problem can be fixed by adding a symbolic 
  link to the Java command in the /usr/bin directory. 
  To do this first execute the command \"which java\" to identify 
  the full path to the Java executable. Next, create a symbolic
  link to this file with the command
  \"ln -sf [path-to-java] /usr/bin/java\" where [path-to-java] is
  the path returned by the \"which\" command.
  "
  exit 1;
 else
  inform_user "- Java command found successfully."
 fi
 JDK_VERSION=`java -version 2>&1`
 java_ver=`echo $JDK_VERSION | head -1 | cut -d" " -f 3 | tr -d \"`
 java_ver_major=`echo $java_ver | cut -d \. -f 1`
 JAVA_VERSION=0

 if [[ $java_ver_major -eq 1 ]]; then
  java_ver_minor=`echo $java_ver | cut -d \. -f 2`
  if [[ $java_ver_minor -ge 8  ]]; then
   JAVA_VERSION=$java_ver_minor
  fi
 elif [[ $java_ver_major -ge 8 ]]; then
  JAVA_VERSION=$java_ver_major
 fi

 # Checking JVM architecture 
 inform_user "DETECTING JVM architecture..."
 JVM_ARCH_DIGIT=`echo $JDK_VERSION | grep 64 | sed 's/\(.*\)\(64\)\(.*\)/\2/'`
 if [ "$JVM_ARCH_DIGIT" != "64" ]; then
  JVM_ARCH_DIGIT="32"
 fi
 inform_user "- $JVM_ARCH_DIGIT bit architecture detected successfully.
"
}

function check_hw_platform() {
 # Checking hardware platform 
 inform_user "DETECTING hardware platform..."
 HW_PLATFORM=`uname -m`
 HW_PLATFORM_DIGIT=`echo $HW_PLATFORM | sed 's/\(.*\)\(64\)\(.*\)/\2/'`
 if [ "$HW_PLATFORM_DIGIT" != "64" ]; then
  HW_PLATFORM_DIGIT="32"
 fi
 inform_user "- Hardware platform detected successfully: $HW_PLATFORM_DIGIT bit
"

}

function check_system() {
 inform_user "******************************************************
*                                                    *
*              Checking the system                   * 
*                                                    *                                                     
******************************************************
"

 check_java
 
 if [[ $JAVA_VERSION -eq 0 ]]; then
  inform_user "
  ERROR: The Java version should be 8 or higher
  "
  exit 1
 fi

 check_hw_platform

 if [ "$HW_PLATFORM_DIGIT" != "$JVM_ARCH_DIGIT" ]; then
  inform_user "Error. JVM architect: $JVM_ARCH_DIGIT is not compatible with hardware architect: $HW_PLATFORM_DIGIT bit($HW_PLATFORM).
  echo Please, uninstall current JVM/JDK and download and install latest JDK $HW_PLATFORM from oracle.com.
  echo Support - support@flashphoner.com, forum - www.flashphoner.com/forums"
  exit 1;
 fi

 inform_user "****************************************************
*                                                  *
*       The system is ready for installation.      *
*                                                  *
*                                      Press ENTER *
****************************************************
"
}

function show_license() {
 if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
  read cont < /dev/tty
 fi

 if [ $ALREADY_INSTALLED != true ] ; then
  for i in `seq 1 6`; do
   PAGE=`cat EULA_PAGE$i`
   echo "$PAGE"
   echo ""
   echo "Press ENTER if you agree to the license terms. Press Ctrl+C to stop installation."
   read cont < /dev/tty
  done
  
  echo "Do you agree to the license terms? [yes/no]"

  until [ "$cont" = "yes" ]
  do
   read cont
   if [ "$cont" = "no" ] ; then
    exit 1;
   fi
  done
  
 fi 
 clear
}

function update_configuration_files() {
 CONF_FILES=(
  "log4j.properties"
  "accounts.xml"
  "database.yml"
  "account.xml"
  "callee.xml"
  "crossdomain.xml"
  "token_keys.properties"
  "dtls-main-cert.pem"
  "dtls-main-key.pem"
  "watchdog.properties"
  "wcs-core.properties"
  "wss.jks.backup"
  "cli-hostkey.pem"
  "dtls0_ua"
 )
 CONF_SUBDIRS=(
  "qa"
  "apps/click-to-call"
 )
 for file in ${CONF_FILES[@]}; do
  if [[ ! -f "$FLASHPHONER_WCS_HOME_FULL/conf/$file" && -f $INSTALLER_DIR/server/conf/$file ]]; then
   cp $INSTALLER_DIR/server/conf/$file $FLASHPHONER_WCS_HOME_FULL/conf
  fi
  if [[ $file == "database.yml" ]]; then
   sed -i 's/9091/8081\/apps/g' $FLASHPHONER_WCS_HOME_FULL/conf/database.yml
  fi
 done
 for subdir in ${CONF_SUBDIRS[@]}; do
  if [[ ! -d "$FLASHPHONER_WCS_HOME_FULL/conf/$subdir" ]]; then
   cp -r $INSTALLER_DIR/server/conf/$subdir $FLASHPHONER_WCS_HOME_FULL/conf
  fi
 done
}

function update_jdk_configuration() {
 WCS_CORE_CONF="$FLASHPHONER_WCS_HOME_FULL/conf/wcs-core.properties"
 if [[ $JAVA_VERSION -ge 11 ]]; then
  old_gc_option=`grep -E "PrintGCDateStamps|UseConcMarkSweepGC|PrintGCDetails|Xloggc|UseCMSInitiatingOccupancyOnly|CMSInitiatingOccupancyFraction" $WCS_CORE_CONF`
  if [ ! -z "$old_gc_option" ]; then
    yes | cp -f $WCS_CORE_CONF "$WCS_CORE_CONF.default"
    sed -i 's/-XX:+UseConcMarkSweepGC//g' $WCS_CORE_CONF
    sed -i 's/-XX:+PrintGCDateStamps//g' $WCS_CORE_CONF
    sed -i 's/-XX:+PrintGCDetails//g' $WCS_CORE_CONF
    sed -i 's/-Xloggc:\/usr\/local\/FlashphonerWebCallServer\/logs\/gc-core-/-Xlog:gc*:\/usr\/local\/FlashphonerWebCallServer\/logs\/gc-core-:time/g' $WCS_CORE_CONF
    # Since JDK 15 more CMS flags are removed #WCS-3094
    if [[ $JAVA_VERSION -ge 15 ]]; then
      sed -i 's/-XX:+UseCMSInitiatingOccupancyOnly//g' $WCS_CORE_CONF
      sed -i 's/-XX:CMSInitiatingOccupancyFraction=[0-9]*//g' $WCS_CORE_CONF
    fi
  fi
  # Since JDK 15 ZPath renamed to AllocateHeapAt #WCS-3094
  if [[ $JAVA_VERSION -ge 15 ]]; then
    sed -i 's/-XX:ZPath/-XX:AllocateHeapAt/g' $WCS_CORE_CONF
  fi
 fi
}

function copy_sdk() {
 cd $INSTALLER_DIR
 cp -R client $FLASHPHONER_WCS_HOME_FULL
 cp -R client2 $FLASHPHONER_WCS_HOME_FULL
 sed -i s/87\.226\.225\.[0-9][0-9]/192.168.1.5/g $FLASHPHONER_WCS_HOME_FULL/client/examples/flashphoner.xml
 sed -i s/192.168.1.[0-9]/$IP_EXTERNAL/g $FLASHPHONER_WCS_HOME_FULL/client/examples/flashphoner.xml
}

function copy_bin() {
 cd $INSTALLER_DIR/server
 mkdir -p $FLASHPHONER_WCS_HOME_FULL/bin
 if [[ $ALREADY_INSTALLED == false ]]; then
  cp -R bin $FLASHPHONER_WCS_HOME_FULL
 elif [[ "$PRODUCT_MAJOR_VERSION" == "$PREVIOUS_MAJOR_VERSION" && "$PRODUCT_MINOR_VERSION" == "$PREVIOUS_MINOR_VERSION" ]]; then
  # Preserve possible scripts tweaks while updating the same product branch #WCS-2615
  for file in bin/*; do
   if [[ "$file" =~ ".sh" ]]; then
    if [ ! -f "$FLASHPHONER_WCS_HOME_FULL/$file" ]; then
     cp -f $file $FLASHPHONER_WCS_HOME_FULL/$file
    fi
   else
     cp -f $file $FLASHPHONER_WCS_HOME_FULL/$file
   fi
  done
 else
  cp -R bin $FLASHPHONER_WCS_HOME_FULL
 fi
 cp -f $INSTALLER_DIR/uninstall.sh $FLASHPHONER_WCS_HOME_FULL/bin
 chmod +x $FLASHPHONER_WCS_HOME_FULL/bin/*.sh
 chmod +x $FLASHPHONER_WCS_HOME_FULL/bin/webcallserver
}

function copy_web() {
 cd $INSTALLER_DIR/server
 cp -R web $FLASHPHONER_WCS_HOME_FULL
}

function copy_lib() {
 cd $INSTALLER_DIR/server
 cp -R lib $FLASHPHONER_WCS_HOME_FULL
}

function copy_tools() {
 cd $INSTALLER_DIR/server
 cp -R tools $FLASHPHONER_WCS_HOME_FULL
 chmod +x $FLASHPHONER_WCS_HOME_FULL/tools/dump_sessions.sh
 chmod +x $FLASHPHONER_WCS_HOME_FULL/tools/certbot-auto
 chmod +x $FLASHPHONER_WCS_HOME_FULL/tools/report.sh
 chmod +x $FLASHPHONER_WCS_HOME_FULL/tools/pcap_*
}

function copy_configs() {
 cd $INSTALLER_DIR/server
 if [[ $ALREADY_INSTALLED == false ]]; then
  cp -R conf $FLASHPHONER_WCS_HOME_FULL
 else
  update_configuration_files
 fi
}

function make_symlink() {
 if [ -L $FLASHPHONER_WCS_HOME ]; then
  rm -f $FLASHPHONER_WCS_HOME
 fi
 ln -sf $FLASHPHONER_WCS_HOME_FULL $FLASHPHONER_WCS_HOME
}

function update_version() {
 cd $INSTALLER_DIR/server
 echo "$VERSION" > $FLASHPHONER_WCS_HOME/conf/WCS.version
}

function copy_files() {
 copy_sdk
 copy_bin
 copy_web
 copy_lib
 copy_tools
 copy_configs
 make_symlink
 update_version
 update_jdk_configuration
 inform_user "- Copying completed."
}

function make_user() {
 check_user
 if [[ $USER_EXISTS == false ]]; then
  useradd --system --home-dir $FLASHPHONER_WCS_HOME --user-group $FLASHPHONER_USER
 fi 
 check_user
 if [[ $USER_EXISTS == true ]]; then
  chown -R $FLASHPHONER_USER $FLASHPHONER_WCS_HOME_FULL
  chgrp -R $FLASHPHONER_USER $FLASHPHONER_WCS_HOME_FULL
 else
  inform_user "Cannot create user $FLASHPHONER_USER, WebCallServer will run as root"
  UPDATE_TO_NON_ROOT=false
 fi
}

function customize_product_config () {
 inform_user "************************************************************************************************
*                                                                                              *
*                             Configuring Flashphoner parameters                               *
*                                                                                              *
*  All parameters will be recorded to the file 'flashphoner.properties'                        *
*  (/usr/local/$PRODUCT/conf/flashphoner.properties).                                    *
*  You can configure it later, if so - just leave all the fields blank (press Enter all time). *
*                                                                                              *
************************************************************************************************
"
 sed -i "s/^ip[[:space:]]*=.*$/ip                     =$IP_EXTERNAL/g" $PRODUCT_CONFIG
 sed -i "s/^ip_local[[:space:]]*=.*$/ip_local               =$IP_LOCAL/g" $PRODUCT_CONFIG

 if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
  echo "- Configuration complete. Your configuration parameters:"
  echo ""
  cat $PRODUCT_CONFIG
  echo ""
  echo "- You can edit it here - $FLASHPHONER_WCS_HOME/conf/flashphoner.properties"
 fi
}

function install() {
 if [ $ALREADY_INSTALLED = true ]; then
  inform_user "Starting $PRODUCT update...
  "
  java -Dcom.flashphoner.fms.AppHome="$FLASHPHONER_WCS_HOME" -cp "./server/lib/*" com.flashphoner.server.license.ExpiredUpdates
  if [ $? -eq 1 ]; then
   exit 1
  fi
 else  
 inform_user "*************************************************************
*                                                           *
*       Starting $PRODUCT installation                      *  
*                                                           *
*************************************************************
"
 fi

 #Request for ip and ip_local settings
 if [[ $ALREADY_INSTALLED != true && "$FLASHPHONER_SILENT" != "-silent" ]]; then
  detect_ip
 fi

 #Creating flashphoner.properties config
 if [ $ALREADY_INSTALLED != true ]; then
  customize_product_config
 fi
 inform_user "COPYING files..."
 
 if [ $ALREADY_INSTALLED = true ]; then
  mv $FLASHPHONER_WCS_HOME-$PREVIOUS_VERSION $FLASHPHONER_WCS_HOME_FULL
  rm -rf $FLASHPHONER_WCS_HOME_FULL/lib
 else
  if [ ! -d $FLASHPHONER_WCS_HOME_FULL ]; then
   mkdir $FLASHPHONER_WCS_HOME_FULL
  fi
 fi
 
 copy_files
}

function startup_enable {
  if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
   echo "Do you want to start WebCallServer at server startup? [yes/no]"
   until [ "$res" = "yes" ]; do
    read res
     if [ "$res" = "no" ]; then
      echo "Use $FLASHPHONER_WCS_HOME/bin/webcallserver start to start server"
      res="yes"
     elif [ "$res" = "yes" ]; then
      if [[ $DAEMON_TYPE == 'systemd' ]]; then
       systemctl enable webcallserver
      else 
       if [ -f /proc/version ]; then
        if grep -e "Red Hat\|centos" /proc/version > /dev/null; then
         chkconfig --add webcallserver
         chkconfig --level 345 webcallserver on
        elif grep -e "[Dd]ebian\|[Uu]buntu" /proc/version > /dev/null; then
         update-rc.d -f webcallserver remove
         update-rc.d webcallserver defaults
        fi    
       else 
        echo "Unable to determine your distribution name"
       fi  
      fi
   	else
   	 echo "Please use yes or no"
   	fi
   done
  fi 
}

function back_to_root() {
  sed -i -e "s/^\(User[ \t]*=\).*\$/\1root/" $SYSTEMD_FILE
  sed -i -e "s/^\(Group[ \t]*=\).*\$/\1root/" $SYSTEMD_FILE
}

function copy_service() {
 cp $INSTALLER_DIR/scripts/systemd/webcallserver.service $SYSTEMD_PATH
 # Revert user back to root if there is such settings,
 # service should always be started from root with sudoing to flashphoner #WCS-3112
 back_to_root
}

function service_setup {
 if [ $ALREADY_INSTALLED != true ]; then
  if [[ $DAEMON_TYPE == 'systemd' ]]; then
   copy_service
   systemctl daemon-reload
  else
   cp -f $FLASHPHONER_WCS_HOME_FULL/bin/webcallserver $INITD_FILE
  fi
  startup_enable
 else
  if [[ $DAEMON_TYPE == 'systemd' ]]; then
     if [ ! -f $SYSTEMD_FILE ]; then
      copy_service
     else
      if [[ $UPDATE_TO_NON_ROOT == true ]]; then
       # Update service to start from user falshphoner #WCS-2913
       srvUser=${FLASHPHONER_USER}
       srvGroup=${FLASHPHONER_USER}
       # Or stay with root privilegies if updated non-interactively #WCS-3056
       if [ "$FLASHPHONER_SILENT" == "-silent" ]; then
        srvUser=root
        srvGroup=root
       fi
       sed -i 's/\[Service\]/\[Service\]\nUser='${srvUser}'\nGroup='${srvGroup}'\nLimitNOFILE='${WCS_FD_LIMIT}'\nAmbientCapabilities=CAP_NET_BIND_SERVICE\n/g' $SYSTEMD_FILE
       sed -i 's@\/var\/run@'${FLASHPHONER_WCS_HOME}'\/bin@g' $SYSTEMD_FILE
      fi
      # Add 143 exit code as success #WCS-3034
      if ! grep -e "^SuccessExitStatus" $SYSTEMD_FILE > /dev/null; then
       sed -i 's/\[Service\]/\[Service\]\nSuccessExitStatus=143\n/g' $SYSTEMD_FILE
      fi
     fi
     systemctl daemon-reload
  elif [ -f $INITD_FILE ]; then
   cp -f $FLASHPHONER_WCS_HOME_FULL/bin/webcallserver $INITD_FILE
  fi
 fi
}

function finish() {
 inform_user "
************************************************************************************************
*                                                                                              *"
 if [ $ALREADY_INSTALLED = true ]; then
  inform_user "*                                   Update complete!                                           *"
  if [ ! -d "$FLASHPHONER_WCS_HOME/logs" ]; then
   mkdir -p $FLASHPHONER_WCS_HOME/logs
  fi	
  echo "$(date +'%Y-%m-%d %H:%M') Updated from $PREVIOUS_VERSION to $VERSION" >> $FLASHPHONER_WCS_HOME/logs/update.log
 else
  inform_user "*                                   Installation complete!                                     *"
 fi
 inform_user "*                             Thank you for trying Flashphoner!                                *
*                                                                                              *
*               Please restart $PRODUCT before start the work.                  				 *
*                  Write us with any questions to support@flashphoner.com                      *
*                                                                                              *
*                                                                                              *
*                                                                                              *
************************************************************************************************
"
 if [ "$FLASHPHONER_SILENT" != "-silent" ]; then
  read cont < /dev/tty
 fi
}

function main() {
 init_vars "$@"
 check_write_permissions
 is_server_installed
 check_user
 if [[ $USER_EXISTS == false ]]; then
  if [[ $ALREADY_INSTALLED == true ]]; then
   UPDATE_TO_NON_ROOT=true
   get_fd_limit
  fi
 fi
 check_system
 if [[ "$FLASHPHONER_SILENT" != "-silent" ]]; then
  show_license
 fi
 install
 make_user
 if [[ $UPDATE_TO_NON_ROOT == true ]]; then
  if grep -e "^WCS_NON_ROOT" $FLASHPHONER_WCS_HOME/bin/setenv.sh > /dev/null; then
   # Switch to non root mode #WCS-2913
   sed -i 's/WCS_NON_ROOT=false/WCS_NON_ROOT=true/g' $FLASHPHONER_WCS_HOME/bin/setenv.sh
  else
   # Add WCS_NON_ROOT declaration if not #WCS-3024
   echo -e "\nWCS_NON_ROOT=true\n" >> $FLASHPHONER_WCS_HOME/bin/setenv.sh
  fi
 fi
 get_hugepages_config
 if [[ $HUGEPAGES_MOUNT_POINT != "" && -d $HUGEPAGES_MOUNT_POINT ]]; then
  # Update hugepages mount point access rights
  chmod o+w $HUGEPAGES_MOUNT_POINT
  RC_LOCAL=/etc/rc.local
  # There is no rc.local on Ubuntu #WCS-3034
  if [[ -e $RC_LOCAL ]]; then
   if [[ -L $RC_LOCAL ]]; then
    RC_LOCAL=$(readlink $RC_LOCAL)
   fi
   sed -i 's@mount -t hugetlbfs -o uid=0 nodev '$HUGEPAGES_MOUNT_POINT'@mount -t hugetlbfs -o uid=0,mode=0757 nodev '$HUGEPAGES_MOUNT_POINT'@g' $RC_LOCAL
  fi
 fi
 service_setup
 finish
}

main "$@"

exit 0
